%
% function [XX   HH  TIME  ISNRs  opt] = BID_NI-UBC(y,opt)
%
% function that implements the BID method of [1,2].
%
% [1] M. S. C. Almeida and  M. A. T. Figueiredo,  . (JOURNAL paper)
%
% [2] M. S. C. Almeida and M. A. T. Figueiredo, "Blind Image Deblurring with Unknown Boundaries Using the
%     Alternating Direction Method of Multipliers", IEEE International Conf. on Image Processing  ICIP2013,
%     Melbourne, Australia, September, 2013. (CONFERENCE paper)
%
% Outputs:  XX       - 3D-vector containing the images estimates at various iterations.
%                      If UBC are considered (instead of assuming cyclcic convolution), the returned images
%                      are sliglty larger than the obserd image (they also cotain the unknown boundaries).
%           HH       - 3D-vector containing the blurring filter estimates along the iterations.
%           INSRs.r1 - INSR adapted meaures (see [1]), computed with a resolution of 1 pixel.
%           INSRs.r3 - INSR adapted meaures (see [1]), computed with a resolution of 1/3 of pixel.
%           TIME     - vector with the processing time up to each iteration of the method.
%           opt      - Strucutre with the method's options.
%
% Inputs:   y   -  observed blurred image.
%          opt  -  method's options, default value in breakets:
%                opt.filter_size (default=[13 13]) - size of the estimated blurring filter (should be odd and larger than the size of the true blur).
%                opt.crt_UBC (default=1) - IF 1, deblurring with Unknown Boundary Conditions.
%                                        - IF 0, deblurring assuming Cyclic Boundary Conditions.
%                opt.Mask_y (default=ones(size(y))) - Binary mask with the size of th eobserved image "y",
%                                                     having 1's in the observed pixels and 0's in the unavailable ones.       
%                opt.lambda0 (default=2) - first values for the regularization parameter lambda.
%                opt.lambda_min (default=1e-15) - last values for the regularization parameter lambda.
%                opt.ration (default=1.7) - goemetric ration for decreasing the regularizatin parameter lambda.
%                opt.max_iter (max_iter=40) - maximun number of iterations (and of lambda values).
%                opt.pp (default=[0.5 0.5 ... 0.5])- vector with the values of the spartity parameter along the iterations.
%                opt.x0 (default=[]) - sharp image, only used for computing the ISNR.
%                opt.nzeros (default=0) - number of extra pixels use to estimate the blurring filter, which will be extracted after the filter estimated.
%                opt.ctr_sym_center (default=0) - IF 1, resctricts the filter to be symetric with respect to its origin.
%                                               - IF 0, does not impose simetry on the estimated blurring filter.
%                opt.ctr_sym_8axis (default=0)  - IF 1, resctricts the filter  to be symetric with respect to 8 axis (horizontal, vertical, 45 and 135 axis).
%                                               - IF 0, does not impose simetry on the estimated blurring filter.
%                opt.ctr_positive (default=1)  - IF 1, restricts the filter to have positive entrances
%                opt.ctr_isnr (default=1) - IF 1, computes the ISNR; requires opt.x0; Stops the method automatically when the ISNR decreases 3 times.
%                                         - IF 0, does not compute the ISNR.
%                opt.ctr_plot (default=1) - IF 1, plots the results along iterations: image estimate, filter estimate, isnr (if selected).
%                                         - IF 0, does not plot any result
%                opt.ctr_print (default=1) - IF 1, prints some results: iteration, time, isnr.
%                                          - IF 0, does not print any result.
%
%
%  opt.Mask_y;  opt.ctr_UBC; opt.h;  opt.x;
%
%
% Mariana S. C. Almeida; 2013.
% Instituto de Telecomunicaes, Lisbon, Portugal
% marianascalmeida@.comgmail


function [XX   HH  TIME  ISNRs  opt] = BID_ADMM_UBC( y , opt )

% % % % INITIALIZATION and VALIDATION:
% INPUTS (default values):
% y:
tic_pre = tic;
XX = [];    HH = []; TIME=[]; ISNRs=[];
if size(y,3)>1,
    fprintf('ERROR: y has more than 2 dimensions.');
    XX = [];    HH = []; TIME=[]; return;
end
%  Options:
try opt.filter_size;   filter_size=opt.filter_size;    catch me, filter_size = [13 13]; end
try opt.ctr_UBC;       ctr_UBC=opt.ctr_UBC;            catch me, ctr_UBC = 1;  end
try opt.Mask_y;        Mask_y = opt.Mask_y;           catch Mask_y =  ones(size(y)); end

try opt.lambda0;    lambda0=opt.lambda0;    catch me, lambda0 = 2;  end
try opt.lambda_min; lambda_min=opt.lambda_min;  catch me, lambda_min =1e-10; end
try opt.ratio;      ratio=opt.ratio;        catch me, ratio = 1.7;  end
try opt.max_iter;   max_iter=opt.max_iter;  catch me, max_iter =50; end
try opt.q_vec;      q_vec=opt.q_vec;              catch me, q_vec = [0.5 0.5]; end

try opt.x0;            x0=opt.x0;             catch me,  end
try opt.ctr_isnr;
    if ~exist('x0'); ctr_isnr = 0; else    ctr_isnr = opt.ctr_isnr; end
catch me, ctr_isnr=0;  end
try opt.ctr_plot;    ctr_plot=opt.ctr_plot;     catch me,  ctr_plot = 1;  end
try opt.ctr_print;   ctr_print=opt.ctr_print;   catch me,  ctr_print = 0; end
try opt.ctr_sym_center;  ctr_sym_center = opt.ctr_sym_center;   catch me, ctr_sym_center = 0; end
try opt.ctr_sym_8axis;   ctr_sym_8axis = opt.ctr_sym_8axis;     catch me, ctr_sym_8axis = 0; end
try opt.ctr_positive;     ctr_positive = opt.ctr_positive;         catch me, ctr_positive = 1; end

ctr_recentra = 1;
max_y = max(y(:));
min_y = min(y(:));

persistent fig1 fig2

% Computing initializations - 1:
l_size = round((filter_size-1)/2);
lx = l_size(1);     ly = l_size(2);
if ctr_UBC, lx_conv = lx;   ly_conv = ly;
else,       lx_conv = 0;    ly_conv = 0;   end
try opt.x;
    x = opt.x;
    % validation:
    if ~( size(x) == size(y) + [2*lx_conv 2*lx_conv] )
        fprintf('ERROR (Deblurring_ADMM_UBC): "opt.xt" should be %dx%d pixels of size.', size(y) + [2*lx 2*ly] ),
    end
catch
    x_in = y;
    x_in = fill_image(x_in,Mask_y);
    x = padarray( padarray( x_in, lx_conv , 'replicate' )' ,ly_conv , 'replicate')' ;
end
[sz1 sz2] = size(x);
try opt.h;  h=opt.h;  catch me,  h = zeros(sz1,sz2); h(1,1)=1;  end
h = ht2h( h2ht(h,lx,ly) , sz1,sz2 );
% Computing initializations - 1 (end)
% INPUTS (end)

% Computing initializations - 2:
MMT = Mask_y;
MTM = padarray( padarray( Mask_y, lx_conv , 0 )' , ly_conv , 0)';
M = @(x) x( 1+lx_conv : end-lx_conv , 1+ly_conv : end-ly_conv ).*Mask_y;
MT = @(x) padarray( padarray( x.*Mask_y, lx_conv , 0 )' , ly_conv , 0 )';
MTy = MT(y);
y_f = fft2(y);

if ctr_isnr, addpath ./ISNR_for_BID, end

lambda_vec = lambda0; % bb - lambda sequence
while lambda_vec(end)>lambda_min;
    lambda_vec(end+1)=lambda_vec(end)/ratio;
end;
if numel(lambda_vec)>max_iter;  lambda_vec(max_iter+1:end)=[];  end

id=0*h; id(1,1)=1;  % identity filter
% Computing initializations - 2 (end)

% REPORT:
TIME(1) = toc(tic_pre);
if (ctr_plot)
    if ~numel(fig1),  fig1 = figure;
    else              figure(fig1);  end
    subplot('position',[0   1-0.57 0.50 0.50]),
    imagesc( [y] ), colormap gray, axis image, title('degraded image '),  drawnow
    subplot('position',[0.5   1-0.57 0.50 0.50]),
    imagesc( [x] ), colormap gray, axis image, title('initial estimate '),  drawnow
end
if ctr_isnr
    if ctr_UBC
        xk = x( 1+lx : end-lx , 1+ly: end-ly );
        hk = ht2h( h2ht(h, lx,ly) , sz1-lx,sz2-lx);
    else
        xk = x; hk = h;
    end
    offset=2;
    ISNRS_dB_res3(1) = ISNR_BID(x,x0,y,  offset, 3);  % resoluao de 1/3 de pixel
    ISNRS_dB_res1(1) = ISNR_BID(x,x0,y,  offset, 1);    % resoluao de 1  pixel
    
    ISNRs.res1(1) = double(ISNRS_dB_res1(1));
    ISNRs.res3(1) = double(ISNRS_dB_res3(1));
end
% REPORT (end)

% BID-loop:  lambda will gradually be decreased
cont_isnr_down = 0;
n_q_vec = length(q_vec);
h0=h;  nISNRdown=0;
grid_x = meshgrid( (-max(lx,ly):max(lx,ly)) );
grid_y = grid_x';

optx=opt; opty=opt;
optx.Mask_y = Mask_y;
opth.Mask_y = Mask_y;

for ii=1:length(lambda_vec)
    
    tic_it = tic;
    
    % new parameters
    lambda = lambda_vec(ii);
    q = q_vec(min(ii,n_q_vec));
    
    %  LEARN IMAGE x:
    optx.ctr_continue = (ii>1);
    optx.x = x;
    [ x ] = findX_ADMM_UBC(y,y_f,MTy,MTM,h,filter_size,lambda,q,optx);
    XX(:,:,ii)=x;
    % LEARN IMAGE x (end)
    
    
    % plot-x:
    tic_plot_x = tic;
    ndisp = 1;
    if ((ctr_plot) ~= 0) && (mod(ii,ndisp) == 0)
        if ~numel(fig2), fig2 = figure;
        else,              figure(fig2),         end
        subplot('position',[0   1-0.57 0.50 0.50]), imagesc(max(min(x,max_y),min_y));   colormap(gray(256)), axis image, %axis off,
        title( strcat('image estimate - it=',num2str(ii)) )
        drawnow;
    end
    time_plot_x = toc(tic_plot_x);
    
    
    % LEARN FILTER h:
    opth.h = h0;
    opth.ctr_continue = (ii>1);
    [h0 ] = findH_ADMM_UBC(y,y_f,MTy,MTM,x,filter_size,lambda,q,opth);
    % Center filter estimate:
    if ctr_recentra
        h1 = h0;
        ht1 = h2ht(h0, lx,ly);
        cm(ii,:) = [ sum(sum(grid_y.*ht1))./sum(sum(ht1))  sum(sum(grid_x.*ht1))./sum(sum(ht1)) ];
        d2 = - round( cm(ii,2) - 0.30*sign(cm(ii,2)) );
        d1 = - round( cm(ii,1) - 0.30*sign(cm(ii,1)) );
        if d1~=0 | d2~=0
            % Shift image x and re-estimate filter h.
            x = circshift( x , max(min([-d1 -d2],2),-2) );
            [h0] = findH_ADMM_UBC(y,y_f,MTy,MTM,x,filter_size,lambda,q,opt);
            
            h1 = h0;
            ht1 = h2ht(h0, lx,ly);
            cm(ii,:) = [ sum(sum(grid_y.*ht1))./sum(sum(ht1))  sum(sum(grid_x.*ht1))./sum(sum(ht1)) ];
        end
        opt.cm=cm;
    end
    h=h0;
    ht =  h2ht( h , lx,ly );
    
    h = ht2h( ht , sz1,sz2 );
    h0=h;   
    h = h/sum(sum(h(:)));
    HH(:,:,ii)=h2ht(h,lx+1,ly+1);
    % Center filter estimate (end)
    % LEARN FILTER h  (end)
    
    TIME(ii+1)=TIME(ii)+toc(tic_it)-time_plot_x;
    
    % REPORT:
    % plot-h:
    if ((ctr_plot) ~= 0) && (mod(ii,ndisp) == 0)
        figure(fig2),
        subplot('position',[0.5 1-0.57 0.50 0.50]), imagesc( h2ht(h,lx+1,ly+1) );
        axis image,  colormap(gray(256)),
        title( strcat( 'filter estimate - it=',num2str(ii) ))
        drawnow;
    end
    XX(:,:,ii)=x;
    % plot-h (end)
    
    % ISNR:
    if ctr_isnr
        if ctr_UBC
            xk = x( 1+lx : end-lx , 1+ly: end-ly );
            hk = ht2h( h2ht(h, lx,ly) , sz1-lx,sz2-lx);
        else
            xk = x; hk = h;
        end
        offset=3;
        ISNRS_dB_res3(ii+1) = ISNR_BID(x,x0,y,  offset, 3);  % resoluao de 1/3 de pixel
        ISNRS_dB_res1(ii+1) = ISNR_BID(x,x0,y,  offset, 1);    % resoluao de 1  pixel
        
        ISNRs.res1(ii+1) = ISNRS_dB_res1(ii+1);
        ISNRs.res3(ii+1) = ISNRS_dB_res3(ii+1);
        
        if ctr_plot
            figure(fig2),
            subplot('position',[0.05   0.05     0.40 0.28]), plot([ISNRS_dB_res1; ISNRS_dB_res3]'),
            title('isnr (base)'); drawnow,
        end
    else
        ISNRS_dB(ii) = -Inf;
    end
    
    if ctr_print & ctr_isnr
        %   if rem(ii,10)==0  fprintf('.'); end
        fprintf( '  it=%d  ISNR(r1,r3)=( %2.2f, %2.2f ), useful time (s): %4.2f \n' , ii,ISNRS_dB_res1(ii),ISNRS_dB_res3(ii), TIME(ii+1) )
    end
    if ctr_isnr
        % stopping:
        if ii>5 & ISNRS_dB_res3(ii)<ISNRS_dB_res3(ii-1) & ISNRS_dB_res1(ii)<ISNRS_dB_res1(ii-1)
            nISNRdown = nISNRdown+1;
        else; nISNRdown = 0;    end
        
        if nISNRdown>3 return, end
    end
    % REPORT (end)
    
end

