%
% Demo_BID_ADMM_UBC: Demo of the BID method proposed in [1] [2], with the possibility of assuming
% Unknown Boundary Conditions (UBC).
%                  Demo applied to a synthetic experiment of cameraman image blurred uncycliclic with a 9x9 uniform blur, at 40dB of BSNR.
%                  The stopping iteration can be selected based on an adapted ISNR measure (in the subfolder ISNR_for_BID),
%                  or on the later measures of whiteness [3,4], whose code is in the subfolder DemoWhiteness.
%
% References on BID:
% [1] M. S. C. Almeida and  M. A. T. Figueiredo,  . (JOURNAL paper)
%
% [2] M. S. C. Almeida and M. A. T. Figueiredo, "Blind Image Deblurring with Unknown Boundaries Using the
%     Alternating Direction Method of Multipliers", IEEE International Conf. on Image Processing  ICIP2013,
%     Melbourne, Australia, September, 2013. (CONFERENCE paper)
%
% References on measures of whiteness:
% [3] M. S. C. Almeida, M. A. T. Figueiredo, "New stopping criteria for iterative blind image deblurring based on
%     residual whiteness measures", IEEE Statistical Signal Processing Workshop - SSP'2011, Nice, France, 2011.
% [4] M. S. C. Almeida, M. A. T. Figueiredo, "Parameter Estimation for Blind and Non-Blind Deblurring Using Residual
%     Whiteness Measures", IEEE Trans. on Image Processing, 2013 (accepted).
%
%
% Mariana S. C. Almeida
% Instituto de Telecomunicaes, Lisbon, Portugal
% marianascalmeida@gmail.com

clear all,
close all

addpath ./images

crt_UBC = 1;    % crt_UBC = 1 - blurring with UBC (extending the image with its border pixels)
% crt_UBC = 0 - blurring assuming Cyclic Boundary Conditions

% LOAD image:
x0 = double(imread('lena256.bmp')); % load image. 
% Other images: 'lena256.bmp' , 'cameraman.tif', 'Satalite.tif'
%               'testpat1.png', 'barbara256.bmp'
x0 = x0 - min(x0(:));  %
x0 = x0./max(x0(:));   % x0 in the range [0,1]

% DEGRADE image:
% - Degradation parameters:
BlurDim = 9;    % filter size (squared): 9x9 pixels
BSNR = 40;      % BSNR level (in dB)
n_filter = 3;   % filter type: 1 -  uniforme
%              2 -  out-of-focus
%              3 -  linear motion at 135
%              4 -  Gaussiano
%              5 -  random, with positive and negative values
%              6 -  nlin motion 1
%              7 -  nlin motion 2 - circle
%              31 to 37 - seven motion filters from Levin et all 2009 
%
%  - Blur:
ht = build_blur( BlurDim , n_filter );    % constructs the blurring filter (9x9)
h = ht2h( ht , size(x0,1) , size(x0,1));  % filter -> h (256x256 pixels)
if crt_UBC
    fsize = round( (BlurDim-1)/2 );                     % fsize - size of each side of the filter
    x02 = padarray( x0, [fsize fsize] , 'replicate' );  % extend the image with its border pixels
    y2 = filter2( flipud(fliplr( ht )) , x02 );         % cyclic convolution
    y = y2( 1+fsize:end-fsize , 1+fsize:end-fsize );    % extracting extra pixels with invalid BC
else
    y = ifft2( fft2(h).*fft2(x0) );  % cyclic convolution
end
%  - Noise:
randn('seed',1);
Py = var(y(:));
sigma = sqrt(Py/10^(BSNR/10));
y_noise = sigma*randn(size(y));
y = y + y_noise;


% method's OPTIONS: see "help_BID_NI_UBC" for full description on the method's options.
opt.filter_size = [BlurDim+6 BlurDim+6];    % opt.BlurDim = 15 - size of the estimated blurring filter (15x15, larger than the real blur of size 9x9)
opt.ctr_UBC = 1;            % opt.crt_UBC = 1 - deblurring with Unknown Boundary Conditions.
% opt.crt_UBC = 0 - deblurring assuming Cyclic Boundary Conditions.
opt.lambda0 = 0.5;        % opt.lambda0 = 2 - first values for the regularization parameter lambda.
opt.lambda_min = 1e-15; % opt.lambda_min = 1e-15 - last values for the regularization parameter lambda.
opt.ratio = 1.5;        % opt.ration = 1.5  - geometric ration for decreasing the regularization parameter lambda.
opt.q_vec = [0.5 0.5];  % opt.pp - vector with the values of the spartity parameter along the iterations.
opt.x0 = x0;            % opt.x0 - sharp image for computing the ISNR.
opt.ctr_isnr = 1;       % opt.ctr_isnr = 1 - computes the (adapted) ISNR; requires opt.x0; Stops the method automatically when the ISNR decreases 4 times.
                        % opt.ctr_isnr = 0 - does not compute ISNR.
opt.ctr_plot = 1;       % opt.ctr_plot = 1 - plots the results along iterations: image estimate, filter estimate, isnr (if computed).
                        % opt.ctr_plot = 0 - does not plot any result
opt.ctr_print = 1;      % opt.ctr_print = 1 - prints some results: iteration, time, isnr (if computed).
                        % opt.ctr_print = 0 - does not prints results.

% % % % % % % % % % % % % % % % % % % % % % %
% CALLING the BID method of [1]:
% see "help_BID_ADMM-UBC" for full discription on the method's options.
% % % % % % % % % % % % % % % % % % % % % % %
fprintf( '\nBID...' )
[XX HH TIME ISNRs opt] = BID_ADMM_UBC( y , opt );
fprintf( 'Done (BID)\n' )

% % % % % % % % % % % % % % % % % % % % % % % % % % %
% Extract the best estimate (at the best iteration) based on:
%  1) adapted measure of ISNR (code in the sub-path ISNR_for_BID")
%  2) whiteness measures of [3,4] (code in the sub-path DemoWhiteness, references in the top of the script)
% % % % % % % % % % % % % % % % % % % % % % % % % %

% - Comuting ISNR measures:
if opt.ctr_isnr
    ISNR_res1 = ISNRs.res1; % isrn computed during BID_QM_UBC, with a resolution of 1 pixel.
    ISNR_res3 = ISNRs.res3; % isrn computed during BID_QM_UBC, with a resolution of 1/3 pixel.
else; fprintf('\n Computing ISNR measures:')
    addpath ./ISNR_for_BID
    for it = 1:size(XX,3)
        x_it = XX(:,:,it);
        offset = 2;
        ISNR_res1(it) = ISNR_BID(x_it,x0,y,  offset, 1);
        ISNR_res3(it) = ISNR_BID(x_it,x0,y,  offset, 3);
    end;
end
fprintf(' ISNR(res1)=%2.5f, ISNR(res3)=%2.5f.',max(ISNR_res1),max(ISNR_res3))

l_size = opt.ctr_UBC.*round( (opt.filter_size-1)/2 );
lx = l_size(1); ly = l_size(2);

% - Comuting measures of whiteness:
compute_whiteness_measures = 1;
if compute_whiteness_measures
    fprintf('\n Computing measures of whiteness:')
    addpath ./DemoWhiteness
    for it = 2:size(XX,3)
        x = squeeze(XX(:,:,it));      % image estimate at iteration "it"
        ht = squeeze(HH(:,:,it-1));   % filter (within the limited support) that was used to estimate x
        h = ht2h(ht, size(x,1) , size(x,2) ); % filter represented with the size of x.
        hx = ifft2(fft2(h).*fft2(x));         % compute Hx, assuming  circular boundary conditions (and an extended image)
        noise_it = y - hx(1+lx:end-lx, 1+ly:end-ly); % residual estimate; the pixels with invalid BC are extracted.
        
        opt = [];  opt.find_local = 1;       % opt.find_local = 1; computes global and local measures of whiteness
        % opt.find_local = 0; only computes global measures of whiteness
        [Measures] = Whiteness_Measures_C(noise_it, opt);
        %  Measures of whiteness, based on:
        R(it) = Measures.R;   %  covariance (M_R in the paper)
        RG(it) = Measures.RG; %  weighted covariance  (M_RG in the paper)
        H(it) = Measures.H;   %  entropy measure (M_H in the paper)
        if opt.find_local
            Rb(it) = Measures.Rb;   %  block covariance  (M_R^b in the paper)
            RGb(it) = Measures.RGb; %  block weighted covariance  (M_RW^b in the paper)
            Hb(it) = Measures.Hb;   %  block entropy  (M_H^b in the paper)
        end
        fprintf('.')
    end
end; fprintf('\n')

% - Extracting the best estimate (best iterations), based on:
%   1) ISNR_res1 , ISNR_res3
[maxi it_isnr1] = max(ISNR_res1);
[maxi it_isnr3] = max(ISNR_res3);
%   2) global measures of whiteness: M , MG , H
if compute_whiteness_measures
    R(1)=R(2); RG(1)=RG(2); H(1)=H(2);
    [maxi it_R] = max(R);
    [maxi it_RG] = max(RG);
    [maxi it_H] = max(H);
    %   3) local measures of whiteness: Mb , MGb , Hb
    if opt.find_local
        Rb(1)=Rb(2); RGb(1)=RGb(2); Hb(1)=Hb(2);
        [maxi it_Rb] = max(Rb);
        [maxi it_RGb] = max(RGb);
        [maxi it_Hb] = max(Hb);
    end
end

% PRINTS the best iterations based on various criteria:
fprintf('\ncriterion:   ISNR_r1  ISNR_r3     R    RG    H')
if opt.find_local, fprintf('     Rb   RGb   Hb '), end
fprintf('\niteration:     %d        %d       %d   %d    %d' , it_isnr1,it_isnr3,it_R,it_RG,it_H);
if opt.find_local, fprintf('    %d   %d    %d' , it_Rb,it_RGb,it_Hb ), end
fprintf('\n');

% PLOTS the recovered images:
scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(3)/6 scrsz(4)*1/3 scrsz(4)/2 scrsz(4)/2])
img_insr3 =  XX(1+lx:end-lx,1+ly:end-ly,it_isnr3);
h2 =  imresize( HH(:,:,max(it_isnr3-1,1)) , 3, 'box'); h2 = h2-min(h2(:)); h2= h2 ./max(h2(:));
img_insr3( 1:size(h2,1) , end-size(h2,2)+1:end) = h2;
imagesc( max(min(img_insr3,1),0) ), colormap gray, axis image,
title(strcat('estimates, it(ISNRres3)=',num2str(it_isnr3),',  ISNRres3=',num2str(ISNR_res3(it_isnr3))))
drawnow
if compute_whiteness_measures & opt.find_local
    figure('Position',[scrsz(3)/6+scrsz(4)/2  scrsz(4)*1/3 scrsz(4)/2 scrsz(4)/2])
    img =  XX(1+fsize:end-fsize,1+fsize:end-fsize,it_RGb);
    h =  imresize( HH(:,:,max(it_RGb-1,1)) , 3, 'box'); h = h-min(h(:)); h = h ./max(h(:));
    img( 1:size(h,1) , end-size(h,2)+1:end) = h;
    imagesc( max(min(img,1),0) ), colormap gray, axis image,
    title(strcat('estimates, it(MGb)=',num2str(it_RGb),',  ISNR-res3=',num2str(ISNR_res3(it_RGb))))
    drawnow
end



