% function [filter BlurDim] = build_blur( BlurDim , n_filter)
%
% Outputs: filter - one out of seven possible blurring filters. 
% Inputs:  BlurDim - dimension of the blurring filter ( BlurDim x BlurDim  pixels ). 
%          n_filter - filter type, from 1 to 7: 
%                       1 -  uniforme
%                       2 -  out-of-focus
%                       3 -  linear motion at 135
%                       4 -  Gaussian
%                       5 -  random, with positive and negative values
%                       6 -  nlin motion 1
%                       7 -  nlin motion 2 - circle         
%                       
%                       31- 1st blurring filter from levin et all
%                       32- 2nd blurring filter from levin et all
%                       33- 3rd blurring filter from levin et all
%                       34- 4th blurring filter from levin et all
%                       35- 5th blurring filter from levin et all
%                       36- 6th blurring filter from levin et all
%                       37- 7th blurring filter from levin et all
%                       38- 8th blurring filter from levin et all
%
%
% Mariana S. C. Almeida
% Instituto de Telecomunicaes, Lisbon, Portugal 
% marianascalmeida@gmail.com
% 
 
function [filter BlurDim] = build_blur( BlurDim , n_filter)
 
switch n_filter  % 7+8 possible filters:
    
    case 1,   % uniforme
        filter = ones(BlurDim,BlurDim);
        
    case 2,   % out_of_focus
        [filter] = fspecial('disk',BlurDim/2);
        
    case 3,   % linear motion
        length = BlurDim;  ang = 135; 
        [filter] =  motion_filter( length, ang);
        
    case 4,   % Gaussian (sig = 2)
        sig = (BlurDim-1)/4; % (sig = 2)
        filter  = Gauss_filter(BlurDim,sig);  
        
    case 5,   % random blur, with positive and negative values
        rand('seed',0)
        filter = rand(BlurDim,BlurDim);
        filter = filter-0.1;
        
    case 6,   % nonlinear motion 1 
        load nlin9x9;       
        filter = nlin9x9{1};        
        
    case 7,   % nonlinear motion 2 - circle
        load circunferencia9x9;  
        filter = circunferencia2{1};
        
    case 31,   load filt01;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);      
        blurring_filter = 'Levin09bFilt01';
    case 32,   load filt02;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);       
        blurring_filter = 'Levin09bFilt02';
    case 33,   load filt03;
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);    
        blurring_filter = 'Levin09bFilt03';
    case 34,   load filt04;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);     
        blurring_filter = 'Levin09bFilt04';
    case 35,   load filt05;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);      
        blurring_filter = 'Levin09bFilt05';
    case 36,   load filt06;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);       
        blurring_filter = 'Levin09bFilt06';
    case 37,   load filt07;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);      
        blurring_filter = 'Levin09bFilt07';
    case 38,   load filt08;   
        filter = fliplr(flipud(f)); 
        BlurDim = size(filter,1);      
        blurring_filter = 'Levin09bFilt08';
        
end;
filter = filter/sum(sum(filter));
 


