% function [x_in x_full time_x cost ISNR mse_full] = findX_ADMM_UBC(y,h,filter_size,lambda,opt)
%
% Estimates the filter "h", using the ADMM with UBC.
%
% OUTPUT:   h - filter estimate.
%
%
% Mariana S. C. Almeida; 2013.
% Instituto de Telecomunicaes, Lisbon, Portugal
% marianascalmeida@.comgmail


function [h] = findH_ADMM_UBC(y,y_f,MTy,MTM,x,filter_size,lambda,q,opt)


tic_pre = tic;

% % % % % INITIALIZATION and VALIDATION:
% % INPUTS (default values):
try opt.Mask_y;         Mask_y = opt.Mask_y;                catch Mask_y =  ones(size(y)); end
try opt.ctr_continue;   ctr_continue = opt.ctr_continue;    catch ctr_continue = 0; end
try opt.ctr_UBC;        ctr_UBC = opt.ctr_UBC;              catch ctr_UBC = 1;      end
try opt.ctr_positive;    ctr_positive = opt.ctr_positive;      catch ctr_positive = 1;  end
try opt.ctr_sym_center;     ctr_sym_center = opt.ctr_sym_center;    catch ctr_sym_center = 0;  end
try opt.ctr_sym_8axis;      ctr_sym_8axis = opt.ctr_sym_8axis;      catch ctr_sym_8axis = 0;  end

l_size = round((filter_size-1)/2);
lx = l_size(1);     ly = l_size(2);
lx_blur  = lx;      ly_blur  = ly;
if ~ctr_UBC,    lx = 0;    ly = 0;   end

% time_h=[]; cost = []; ISNR =[]; mse_full=[];


if ~(size(MTy) == size(y) + [2*lx 2*ly])
    fprintf('ERROR (findX_ADMM_UBC): "MTy" should be %dx%d pixels of size (size y: (%dx%d), filter_size: %dx%d).\n', size(y) + [2*lx 2*ly], size(y),  filter_size),
    return,
elseif ~(size(y_f) == size(y))
    fprintf('ERROR (findX_ADMM_UBC): "y_f" (fft2 of y) should be %dx%d pixels of size (size y: (%dx%d)).\n', size(y) , size(y)),
    return,
elseif ~(size(MTM) == size(y) + [2*lx 2*ly])
    fprintf('ERROR (findX_ADMM_UBC): "MTM" should be %dx%d pixels of size (size y: (%dx%d), filter_size: %dx%d).\n', size(y) + [2*lx 2*ly], size(y),  filter_size),
    return,
end

try opt.h;
    h = opt.h;
    if ~( size(h) == size(y) + [2*lx 2*ly] )
        fprintf('ERROR (findX_ADMM_UBC): "opt.h" should be %dx%d pixels of size.\n', size(y) + [2*lx 2*ly] ),
        return
    end
catch,
    h = zeros(size(x));    h(1,1) = 1;
end

if ~(size(x) == size(h))
    fprintf('ERROR (findX_ADMM_UBC): "x" and "h" should be have the same size.\n'),
    return,
end
persistent max_iter     ctr_print   ctr_plot     fig3
persistent ctr_adapt    t_up      t_down       ro
persistent n_f   f_f     fT_f    sum_fTf_f   sfilter_in  sfilter
persistent sz1   sz2

miu_xh = 0.01;
miu_ph = 0.1;

if ~ctr_continue
    ctr_adapt = 1;
    ctr_print = 0;
    ctr_plot = 0;
    max_iter = 15;
    
    ctr_adapt = 1;
    t_up = 2;
    t_down = 2;
    ro = 10;
    
    [sz1 sz2] = size(x);
end


% % % % % ADMM INITIALIZATIONS:
x_f = fft2(x);
x_f_conj = conj(x_f);
xTx_f = x_f_conj.*x_f;
h_f = fft2(h);
xh_f = x_f.*h_f;
xh = ifft2(xh_f);

g_xh = xh;
g_ph = h;   % h2ht(h,fsize);
d_xh = 0*g_xh;
d_ph = 0*g_ph;
% % % % % ADMM INITIALIZATIONS (end)


time_pre  = toc(tic_pre);
times_h(1) = time_pre;

for iter=1:max_iter
    
    tic_iter = tic;
    
    h_old = x;
    g_xh_old = g_xh;
    g_ph_old = g_ph;
    
    % H-step:
    h_f = ( miu_xh.*x_f_conj.*(fft2(g_xh+d_xh)) + miu_ph.*fft2(g_ph+d_ph) ) ./ ( miu_xh.*xTx_f + miu_ph );
    h =  ifft2( h_f );
    % h-setp (end)
    
    
    % u-setps:
    xh = ifft2(h_f.*x_f);
    g_xh = (MTy + miu_xh*(xh - d_xh)) ./ (miu_xh + MTM);
    
    
    g_ph_ht =  h2ht( h-d_ph , lx_blur,ly_blur );
    if ctr_sym_center,      g_ph_ht = Sym_center(g_ph_ht);
    elseif ctr_sym_8axis,   g_ph_ht = Sym_8axis(g_ph_ht);       end
    if ctr_positive          g_ph_ht = g_ph_ht.*(g_ph_ht>0);     end
    g_ph = ht2h( g_ph_ht , sz1,sz2 );
    % u-setps (end)
    
    % d-steps:
    d_ph_old=d_ph;
    d_xh_old=d_xh;
    
    r_xh = xh - g_xh;
    d_xh = d_xh - r_xh;
    
    r_ph = h - g_ph;
    d_ph = d_ph - r_ph;
    % d-steps (end)
    
    times_h(iter+1) = times_h(iter) + toc(tic_iter);
    
    if ctr_adapt
        
        % Adapt miu_ph:
        s_ph = miu_ph.*(-1)*( (g_ph-g_ph_old) );
        rn_ph = norm(r_ph(:),'fro');
        sn_ph = norm(s_ph(:),'fro');
        if rn_ph > ro*sn_ph,       miu_ph = t_up * miu_ph;
        elseif sn_ph > ro*rn_ph;   miu_ph = miu_ph / t_down;
        end
        
        % Adapt miu_xh:
        s_xh = miu_xh.*(-1)*ifft2( (x_f_conj).*fft2(g_xh-g_xh_old) );
        sn_xh = norm(s_xh(:),'fro');
        rn_xh = norm(r_xh(:),'fro');
        if rn_xh > ro*sn_xh,       miu_xh = t_up * miu_xh;
        elseif sn_xh > ro*rn_xh;   miu_xh = miu_xh / t_down;
        end
        
        
    end
    
  
    
end

end


