% function [x] = findX_ADMM_UBC(y,h,filter_size,lambda,opt)
%
% Estimates the image "x", using the ADMM with UBC.
%
% OUTPUT:   x - image estimate.
%
%
% Mariana S. C. Almeida; 2013.
% Instituto de Telecomunicaes, Lisbon, Portugal
% marianascalmeida@.comgmail

function [x ] = findX_ADMM_UBC(y,y_f,MTy,MTM,h,filter_size,lambda,q,opt)

tic_pre = tic;

% % % % % INITIALIZATION and VALIDATION:
% % INPUTS (default values):
try opt.Mask_y;         Mask_y = opt.Mask_y;                catch Mask_y =  ones(size(y)); end
try opt.ctr_continue;   ctr_continue = opt.ctr_continue;    catch ctr_continue = 0; end
try opt.ctr_UBC;        ctr_UBC = opt.ctr_UBC;              catch ctr_UBC =  1; end

l_size = round((filter_size-1)/2);
lx = l_size(1);     ly = l_size(2);
if ~ctr_UBC,    lx = 0;    ly = 0;   end

% time_x=[]; cost = []; ISNR =[]; mse_full=[];

if ~(size(MTy) == size(y) + [2*lx 2*ly])
    fprintf('ERROR (findX_ADMM_UBC): "MTy" should be %dx%d pixels of size (size y: (%dx%d), filter_size: %dx%d).\n', size(y) + [2*lx 2*ly], size(y),  filter_size),
    return,
elseif ~(size(y_f) == size(y))
    fprintf('ERROR (findX_ADMM_UBC): "y_f" (fft2 of y) should be %dx%d pixels of size (size y: (%dx%d)).\n', size(y) , size(y)),
    return,
elseif ~(size(MTM) == size(y) + [2*lx 2*ly])
    fprintf('ERROR (findX_ADMM_UBC): "MTM" should be %dx%d pixels of size (size y: (%dx%d), filter_size: %dx%d).\n', size(y) + [2*lx 2*ly], size(y),  filter_size),
    return,
end

try opt.x;
    x = opt.x;
    if ~( size(x) == size(y) + [2*lx 2*ly] )
        fprintf('ERROR (findX_ADMM_UBC): "opt.x" should be %dx%d pixels of size.\n', size(y) + [2*lx 2*ly] ),
        return
    end
catch,
    x_in = y;
    x_in = fill_image(x_in,Mask_y);
    x = padarray( padarray( x_in, lx , 'replicate' )' ,ly , 'replicate')' ;
    x_init= x;
end

if ~(size(x) == size(h))
    fprintf('ERROR (findX_ADMM_UBC): "x" and "h" should be have the same size.\n'),
    return,
end
persistent max_iter     ctr_print   ctr_plot     fig1
persistent ctr_adapt    t_up      t_down       ro
persistent n_f   f_f     fT_f    sum_fTf_f   sfilter_in  sfilter
persistent sz1   sz2

miu_hx = 0.5;
miu_f_x = 2*lambda;

if ~ctr_continue
    ctr_adapt = 1;
    ctr_print = 0;
    ctr_plot = 0;
    max_iter = 20;
    
    ctr_adapt = 1;
    t_up = 2;
    t_down = 2;
    ro = 10;
    
    [sz1 sz2] = size(x);
    
    sfilter = build_sfilter(x);
    n_f = size(sfilter,3);
    sum_fTf_f = zeros(size(x));
    f_f = zeros(sz1,sz2,n_f);
    fT_f = f_f;
    for i = 1:n_f
        f_f(:,:,i) = fft2(sfilter(:,:,i));
        fT_f(:,:,i) = conj(f_f(:,:,i));
        sum_fTf_f = sum_fTf_f + (fT_f(:,:,i).*f_f(:,:,i));
    end
    
end


% % % % % ADMM INITIALIZATIONS:
h_f = fft2(h);
h_f_conj = conj(h_f);
hTh_f = h_f_conj.*h_f;
x_f = fft2(x);
hx_f = x_f.*h_f;
hx = ifft2(hx_f);

g_hx = hx;
for i = 1:n_f,
    Fx(:,:,i) = ifft2( (f_f(:,:,i)).* x_f );
end;
g_f_x = Fx;
d_hx = 0*g_hx;
d_f_x = 0*g_f_x;
% % % % % ADMM INITIALIZATIONS (end)

time_pre  = toc(tic_pre);
time_x(1) = time_pre;
for iter=1:max_iter
    
    tic_iter = tic;
    
    x_old = x;
    g_hx_old = g_hx;
    g_f_x_old = g_f_x;
    
    % X-step:
    auxi_f_x_f = zeros(size(x));
    for i = 1:n_f
        auxi_f_x_f = auxi_f_x_f + ( (fT_f(:,:,i)).*fft2( ( g_f_x(:,:,i)  + d_f_x(:,:,i) )) ) ;
    end
    auxi_hx_f = h_f_conj.*( fft2(g_hx + d_hx) );
    x_f = ( miu_hx.*auxi_hx_f + miu_f_x*auxi_f_x_f )./( miu_hx.*hTh_f + miu_f_x*sum_fTf_f );
    x = ifft2( x_f );
    % x-setp (end)
    
    
    % u-setps:
    hx = ifft2(h_f.*x_f);
    g_hx = (MTy + miu_hx*(hx - d_hx)) ./ (miu_hx + MTM);
    
    for i = 1:n_f
        Fx(:,:,i) = ifft2( (f_f(:,:,i)).*x_f ) ;
        auxi_f_x(i,:,:) = (Fx(:,:,i))  -  (d_f_x(:,:,i));
    end
    g_f_x = reshape( generalized_shrinkage( auxi_f_x , lambda./miu_f_x  , q ) , n_f ,sz1,sz2);
    g_f_x = permute( g_f_x , [2 3 1]);
    % u-setps (end)
    
    % d-steps:
    r_hx = hx - g_hx;
    d_hx = d_hx - r_hx;
    
    r_f_x = Fx - g_f_x;
    d_f_x = d_f_x - r_f_x;
    % d-steps (end)
    
    time_x(iter+1) = time_x(iter) + toc(tic_iter);
    
    if ctr_adapt
        
        % Adapt miu_f_x:
        fT_dg = 0*d_f_x(:,:,1);
        for i = 1:n_f
            fT_dg = fT_dg - ifft2( (fT_f(:,:,i)).*fft2( (g_f_x(:,:,i) - g_f_x_old(:,:,i)) ) );
        end
        s_f_x = miu_f_x .* fT_dg;
        
        rn_f_x = norm(r_f_x(:),'fro');
        sn_f_x = norm(s_f_x(:),'fro');
        if rn_f_x > ro*sn_f_x,         miu_f_x = t_up * miu_f_x;
        elseif sn_f_x > ro*rn_f_x,     miu_f_x = miu_f_x / t_down;
        end
        
        % Adapt miu_hx:
        s_hx = miu_hx.*(-1)*ifft2( (h_f_conj).*fft2(g_hx-g_hx_old) );
        sn_hx = norm(s_hx,'fro');
        rn_hx = norm(r_hx(:),'fro');
        if rn_hx > ro*sn_hx,       miu_hx = t_up * miu_hx;
        elseif sn_hx > ro*rn_hx;   miu_hx = miu_hx / t_down;
        end
        
    end
    
end

end


